// Term.h
//
// This class represents a logical predicate. Its main use is for
// representing Prolog clauses.
//
// Author : Martijn van Welie
//

#ifndef TERM_H
#define TERM_H

extern "C" {
#include <sicstus/sicstus.h>
}

const MAX_TERM_ARGS 	= 10;
const MAX_BUF_LEN	= 256;

class Term {
public:
	Term();					// Empty constructor
	Term(char*,int arity,char *module);	// Normal constructor
	Term(const Term&);			// Copy constructor

	virtual 	~Term();		
	void		Initialize();

	// Extract term information
	const char*	Name() const;
	char*		toString(void);
	int		Arity() const;
	const char*	moduleName() const;
	int		argCount() const;

	// Adding a new arguments to the term
	int	put(long l);
	int	put(double d);
	int 	put(char *name);
	int	put(Term &t);
	int 	putVariable();		// Add a new variable
	int	putVariable(int index);	// Reuse variable of the index-th 

	// Getting argument values
	int 	getInteger(int index,long &l);
	int	getFloat(int index,double &d);
	char*	toString(int index);
	char*	termToString(SP_term_ref t);

	// Data members
	SP_pred_ref	m_pred;
	SP_term_ref	m_argv[MAX_TERM_ARGS];
	int		m_argc;
	int		m_arity;
	char		m_moduleName[MAX_BUF_LEN];
	char 		m_name[MAX_BUF_LEN];
	int		m_varCount;
	int		m_vars[MAX_TERM_ARGS];
protected:
	char		m_buf[MAX_BUF_LEN];
private:
};

#endif
