#ifndef HANDLER_H
#define HANDLER_H

#include <hush/export.h>
#include <hush/obscure.h>

#ifdef __GNUG__
#define NOHANDLER
#endif

#include <hush/api/option.h>

//Unfortunately, OK for Tcl is 0,
//whereas OK for the simulation packages is 1. OK?

#undef OK
#define OK 0
#undef ERROR
#define ERROR 1

 // TODO: is a handler or thread or what
 // include <hush/thread.h>

class handler;
class event;
class kit;

class term;
class dvstring;
class iter_type;
class dictionary_type;
class dvlist_type;
class garbage_type;
class dvbinding;
class clt_type;
class srv_type;
class vm_type;

class IO;



class HUSH_API handler : public obscure { 	// man:handler(4)
static int _no;                    // to keep count of handlers
public:

enum  { implicit = 0, explic }; // modes -- GNU does not allow explicit!


	handler();
	virtual ~handler();
	

public: //@ dispatching
	
	virtual event* dispatch(event* e = 0);
	virtual int operator()();

protected: //@ tk & _event
    event* _event;
    kit* tk;

public: //@ default methods - originally introduced for canvas

virtual void press( event&  ) { }
virtual void release( event&  ) { }
virtual void keypress( event&  ) { }
virtual void keyrelease( event&  ) { }
virtual void motion( event&  ) { }
virtual void enter( event&  ) { }
virtual void leave( event&  ) { }
virtual void other( event& );

// Access to the (local) kit:

virtual kit* thekit() const; // OBSOLETE

public: //@ info

	virtual char* name() const;    	//- get handler's command name
	virtual void rename(const char*); //- (re)set handler's command name
	
	virtual char* description() const; //- Explaining text
	
	virtual char* mimetype() const; //- when acting as a plugin
	
	
	virtual char* category() const; //- handler's kind 
	
	virtual char* info() const;     //- 

public: //@ io

virtual IO& out(const char* opts=0) const;
virtual IO& err(const char* opts=0) const;
virtual IO& in(const char* opts=0)  const;

public: //@ modalities

virtual int mode() const;
virtual int mode(int);
//virtual int mode(const char*);

public: //@ dependent handlers

virtual void dependent(handler* e); // to define dependent handlers 
virtual void remove(handler* e); // to remove dependent handlers
virtual int contains(handler* e); // to check for cycles
virtual void process(event* e = 0); // to activate dependent handlers

public: //@ garbage collection

virtual void _declare(const handler* h) const; // for the future

void _cleanup(const char* opts = 0) const; // deletes garbage!

void _register(const obscure* h) const; // if we don't know anything
void _register(const handler* h) const; // for collection
void _register(const term* p) const; // idem
void _register(const dvstring* p) const; // idem
void _register(const dvlist_type* p) const; // idem
void _register(const iter_type* p) const; // idem
void _register(const dictionary_type* p) const; // idem
void _register(const garbage_type* p) const; // idem
void _register(const char* p) const; // idem
void _register(const dvbinding* p) const; // idem
void _register(const srv_type* p) const; // idem
void _register(const clt_type* p) const; // idem
void _register(const vm_type* p) const; // idem

public: //@ Options and Resources 


virtual handler_option& operator[](const char* key);

virtual void option(const char* k, const char* v);
virtual char* option(const char* k);

virtual int _option(const char*);
virtual void _option(const char*,int);
     
virtual handler* _resource(const char*);
virtual void  _resource(const char*, handler*);

private: //@ some of these could have been functions, alas ...
    char* _name;
    void*  _garb; // to release you from garbage collection - be careful

public: //@ to emulate kit and event

virtual long eval(const char* cmd);
virtual char* evaluate(const char* cmd);

virtual char* result(long id=0);
virtual void result(const char* s);

public: //@ debug & warn
    virtual void _debug(const char* s, const char* msg = "") const;
    virtual void _warning(const char* s, const char* msg = "") const;
    virtual void* _error(const char* s, const char* msg = "") const; //0
    virtual void _obsolete(const char* s, const char* msg = "") const;


protected: 
    virtual void _write(const char* s, const char* msg = "") const;

public: //@ to create meta handlers
    
protected:    virtual void _reparent(handler* h); // to outer context
protected:    virtual handler* _redirect(handler* h); // to inner self

public:       virtual void _alias( handler* caller = 0 ); // for 'global' body

private: //@ to support 'layered' handler components

	handler* _this() const; // _context() or this
  	int _exists() const;    // is there a context?

protected:
  	handler* _body() const; // the implementation
	handler* _context() const; // non-virtual
	handler* _self() const;    // idem

public: virtual int invariant() const; // to check the handler's state


private: //@ ...
void* _state;

public: //@ utilities 

char* flatten(int argc, char* argv[]) const;
char* quote(int argc, char* argv[]) const;
char* gensym(const char* s=0) const;

char* unquote(const char* s) const; // for pl

char* filein(const char* f=0);
void fileout(const char*, const char*);

virtual void obsolete(const char* msg = "HANDLER") const; // is obsolete 
virtual int verify() const;

private: //@ options

public: //@ defaults
static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static handler* lookup(const char*);
static void destroy();


public:
static IO* _out;
static IO* _err;
static IO* _in;

static int initialized;
static int initialize();
};

#endif
