#ifndef KIT_H
#define KIT_H


#include <hush/export.h>

#include <hush/handler.h>
#include <hush/term.h>

class dvbinding;
class widget;
class factory;
class dot;
class parser;
class scanner;
class formatter;

class HUSH_API kit : public handler {

public:

enum { tcl = 0, dlp, python, unknown };

kit(const char* = "tcl");
kit(kit*);
virtual ~kit();

public: //@ experimental

virtual int mode(int fl);
virtual int mode() const;

public: //@ Session management

virtual int load(const char* libs);
virtual kit* check(const char* kind);


public: // Query -- logic, pl, dlp etc

virtual term* query(const char* pred, int qmode=0);  // is mode dependent !
virtual int  consult(const char* pred);

public: //@ Tcl/Tk - a method interface for the interpreter

virtual long eval(const char* cmd);
virtual char* evaluate(const char* cmd);

virtual char* result(long id=0);
virtual void result(const char* s);

virtual int source(const char* f);

virtual void _assert(const char* x, const char* opt="");
virtual void _retract(const char* x, const char* opt="");

virtual int error(); // N E W
virtual void error(const char*); // N E W


virtual void wait(int n) { after(n); }
virtual void after(int n, handler* h = 0);
virtual void* token() const;  // Last TimerToken

virtual char* send(const char* it, const char* cmd);
virtual char* selection(const char* options="") const;
virtual void update(const char* options="");

virtual widget* root() const; 
virtual widget* pack(widget* w, const char* options = "-side top -fill x");
virtual widget* pack(const char* w, const char* options = "-side top -fill x");

virtual void trace(int i = 1);
virtual void notrace();
virtual int  tracelevel() const;

virtual void quit();

public: //@ options


virtual void option(const char* k, const char* v);
virtual char* option(const char* k);

public: //@ advanced: bindings

enum {readable=1, writable=2, exception=4}; // jrvosse

virtual dvbinding* bind(handler* h); // anonymous
virtual dvbinding* bind(const char* name, handler* h);
virtual dvbinding* bind(int fd, handler* h, int mask=readable); // jrvosse

virtual void unbind(int fd);
virtual void unbind(char* name, char* newname = "");


public: //@ obsolete

virtual int safe() const;
virtual kit* slave(char* options = "");


// Miscelaneous:

virtual void* interp() const; // the interpreter, whatever it is
virtual void* tkwin(const char* path=0) const;  // returns the (root0 window
virtual void* display() const; // return the display

// System-level:

virtual int map(widget*,int i = 1) { return i; } // to map or unmap widgets

// You can play numerous tricks with events - <b>obsolete</b>

virtual event* get() const;
virtual void put(event* p, int fl = 0); // to update events

// Experimental -- role switching

kit* become(char* s, kit* k = 0);
void restore(kit* k = 0);

// Experimental

virtual void font(char*);
virtual int alias( char*, handler* );

public: //@ special kit and parser factory factory

virtual class kit* _kit(const char* k = "tk");
virtual class parser* parser(const char* k = "term");
virtual class scanner* scanner(const char* k = "term");
virtual class formatter* formatter(const char* k = "term");

public: //@ factory + dot 

virtual class button* button(const char* kind = 0);
virtual class button* button(widget* r, const char* p, const char* opts = "");

virtual class checkbutton* checkbutton(const char* kind = 0);
virtual class checkbutton* checkbutton(widget* r, const char* p, const char* opts = "");

virtual class radiobutton* radiobutton(const char* kind = 0);
virtual class radiobutton* radiobutton(widget* r, const char* p, const char* opts = "");

virtual class menu* menu(const char* kind = 0);
virtual class menu* menu(widget* r, const char* p, const char* opts = "");

virtual class menubutton* menubutton(const char* kind = 0);
virtual class menubutton* menubutton(widget* r, const char* p, const char* opts = "");

virtual class menubar* menubar(const char* kind = 0);
virtual class menubar* menubar(widget* r, const char* p, const char* opts = "");

virtual class message* message(const char* kind = 0);
virtual class message* message(widget* r, const char* p, const char* opts = "");

virtual class entry* entry(const char* kind = 0);
virtual class entry* entry(widget* r, const char* p, const char* opts = "");

virtual class listbox* listbox(const char* kind = 0);
virtual class listbox* listbox(widget* r, const char* p, const char* opts = "");

virtual class frame* frame(const char* kind = 0);
virtual class frame* frame(widget* r, const char* p, const char* opts = "");

virtual class toplevel* toplevel(const char* kind = 0);
virtual class toplevel* toplevel(widget* r, const char* p, const char* opts = "");

virtual class scrollbar* scrollbar(const char* kind = 0);
virtual class scrollbar* scrollbar(widget* r, const char* p, const char* opts = "");

virtual class label* label(const char* kind = 0);
virtual class label* label(widget* r, const char* p, const char* opts = "");

virtual class canvas* canvas(const char* kind = 0);
virtual class canvas* canvas(widget* r, const char* p, const char* opts = "");

virtual class text* text(const char* kind = 0);
virtual class text* text(widget* r, const char* p, const char* opts = "");

virtual class filechooser* filechooser(const char* kind = 0);
virtual class filechooser* filechooser(widget* r, const char* p, const char* opts = "");

virtual class browser* browser(const char* kind = 0);
virtual class browser* browser(widget* r, const char* p, const char* opts = "");

public: // but advanced!
class dot* _dot;
virtual class dot* dot(const char* kind = 0);
class factory* _factory;
virtual class factory* factory(const char* kind = 0);

//</slide>

private:
int _mode;
kit* tk;

//<hr>

public: //@ defaults

static kit* _default;
static void* _create;

static void set_default(kit* x);// { _default = x; }
static kit* get_default();// { return _default; }

static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static kit* lookup(const char*);
static void destroy();

};

extern HUSH_API kit* thekit();

#endif
