#ifndef _hush_corba_orb_ooc_ior_h
#define _hush_corba_orb_ooc_ior_h

#include <OB/Basic.h>
#include <OB/Except.h>
#include <OB/Template.h>
#include <OB/Declarations.h>
#include <OB/Util.h>
#include <OB/IOP.h>
#include <OB/Object.h>
#include <OB/GIOP.h>
#include <OB/IIOP.h>
#include <OB/OCI.h>
#include <OB/Net.h>
//#include <OB/SSLIOP.h>
//#include <OB/OCI_SSL.h>
//#include <OB/OBSSL.h>

CORBA_ULong print_ior(ostream& out, const IOP_IOR& ior);


void print_ior(ostream& out, const char* s)
{
    assert_nca(s, OBNCANullString);
    
    if(strncmp(s, "IOR:", 4) != 0)
	throw CORBA_INV_OBJREF();
    
    s += 4;
    
    CORBA_Octet* const buf = OBAsciiToOctets(s);
    const CORBA_Octet* oct = buf;
    
    CORBA_Boolean endian;
    OBUnmarshal(endian, oct, false);
    
    IOP_IOR ior;
    OBUnmarshal(ior, oct, endian != OBEndian);
    
    delete [] buf;

    out << "byteorder: " << (endian ? "little" : "big") << " endian\n";

    print_ior(out, ior);
}


CORBA_ULong print_ior(ostream& out, const IOP_IOR& ior)
{
    CORBA_ULong count = 0;
    for(CORBA_ULong i = 0 ; i < ior.profiles.length() ; i++)
    {
	if(ior.profiles[i].tag == IOP_TAG_INTERNET_IOP)
	{
	    count++;

	    //
	    // Get the IIOP profile body
	    //
	    const CORBA_Octet* coct = ior.profiles[i].profile_data.data();
	    CORBA_Boolean endian;
	    OBUnmarshal(endian, coct, false);
	    IIOP_ProfileBody body;
	    OBUnmarshal(body, coct, endian != OBEndian);

	    //
	    // Print info
	    //
	    out << "IIOP profile #" << count << ":\n";
	    out << "iiop_version: " << int(body.iiop_version.major) << '.'
		<< int(body.iiop_version.minor) << '\n';
	    out << "host: " << body.host << '\n';
	    out << "port: " << body.port << '\n';
	    out << "object_key:\n";
	    OBPrintOctets(out, body.object_key.data(),
			  body.object_key.length());
	}
	/*
        else if(ior.profiles[i].tag == SSLIOP_TAG_SSL_SEC_TRANS)
        {
            count++;

	    //
	    // Get the SSL profile body
	    //
	    const CORBA_Octet* coct = ior.profiles[i].profile_data.data();
	    CORBA_Boolean endian;
	    OBUnmarshal(endian, coct, false);
            SSLIOP_SSL body;
	    OBUnmarshal(body, coct, endian != OBEndian);

	    //
	    // Print info
	    //
	    out << "SSL profile #" << count << ":\n";
            out << "target_supports: " << body.target_supports << '\n';
            out << "target_requires: " << body.target_requires << '\n';
	    out << "port: " << body.port << '\n';
        }*/
    }

    return count;
}
#endif
