#ifndef _hush_midi_file_h
#define _hush_midi_file_h

#include <hush/export.h>
#include <stdio.h>
#include <hush/midi/handler.h>

class HUSH_API midi_file  : public midi_handler {
char* _file;
FILE* fp_in;
FILE* fp_out;
int _division;
int _tempo;
int _tracks;
int _track;
protected:
int _in_seconds;
int in_seconds();

public: //@ info

virtual FILE* in_file();
virtual FILE* out_file();


int division();
int tracks();
int track();
int tempo();
double seconds();
char* file();
long time();


int write(const char* f, int format=0, int ntracks=1, int division=480); 
int open(const char* f, const char* mode="w");
int close();
midi_file(); 

public: //@ process write
void mfwrite(int format, int ntracks, int division, FILE* fp); 
int mf_write_midi_event( unsigned long delta_time, unsigned int chan,
	unsigned int type, unsigned char *data, unsigned long size);
int mf_write_meta_event( unsigned long delta_time,
	unsigned char type, unsigned char *data, unsigned long size);
void mf_write_tempo( unsigned long tempo );



private:
void mf_write_track_chunk(int which_track, FILE* fp);
void mf_write_header_chunk(int format, int ntracks, int division);
unsigned long mf_sec2ticks(float  secs, int division, unsigned int tempo);
void WriteVarLen( unsigned long value);
float mf_ticks2sec( int division, unsigned int tempo, unsigned long ticks);
void write32bit( unsigned long data );
void write16bit(int data);
int eputc( unsigned char c );

public: //@ process read

void mfread(FILE* fp = 0);
void midifile(const char* f = 0, const char* out = 0);

public: //@ write

void write_header(int format, int ntracks, int division);
void write_tempo(int x);
int begin_track(int trck=0);
int end_track();
void write_event(int delta, int chan, int type, const  char* data, int size);
void write_meta(int delta, int type, const  char* data, int size);
private:
unsigned long trkhdr;
unsigned long trklength;
long offset;
long place_marker;
int _in_track;
int _channel;
int _trackno;

protected:
void mferror(char *s);
int readtrack();
int readheader();

private:
int readmt(char*);
int egetc();

long readvarinum();
long read32bit();
long to32bit(int c1, int c2, int c3, int c4);
int read16bit();
int to16bit(int c1, int c2);
char *msg();

void biggermsg();

void msginit();
int msgleng();
void msgadd(int c);

void badbyte(int c);

void metaevent(int type);
void sysex();

void chanmessage(int status, int c1, int c2);


private:
int Mf_nomerge;
long Mf_currtime;

private: // more private

long Mf_toberead;
long Mf_numbyteswritten;
};

#endif
