#ifndef _hush_midi_handler_h
#define _hush_midi_handler_h

#include <hush/export.h>
#include <hush/handler.h>

#include <stdio.h>
#include <hush/midi/midi.h>

class HUSH_API midi_handler : public handler {
protected:
midi_handler* _master;
midi_handler* _handler;
int _in_seconds;
public: //@ handler

midi_handler(); 
virtual int read(const char* f = 0, const char* out = 0);
virtual int write(const char* f = 0,int format=0, int trcks=1, int divis=480);
virtual int open(const char* f = 0, const char* mode="w");
virtual int close();

public: //@ util
virtual char* note(int v);

public: //@ info

virtual FILE* in_file();
virtual FILE* out_file();

virtual int division();
virtual int tracks();
virtual int track();
virtual int tempo();
virtual double seconds();
virtual char* file();
virtual long time();
virtual int in_seconds();

public: //@ read
virtual int midi_getc();
virtual int midi_error( char *s );
virtual int midi_header(int format, int ntrks, int ldivision) ;
virtual int midi_trackstart();
virtual int midi_trackend();
virtual int midi_noteon(int chan, int pitch, int vol);
virtual int midi_noteoff(int chan, int pitch, int vol);
virtual int midi_pressure(int chan, int pitch, int press);
virtual int midi_parameter(int chan, int control, int value);
virtual int midi_pitchbend(int chan, int msb, int lsb);
virtual int midi_program(int chan, int program);
virtual int midi_chanpressure(int chan, int press);
virtual int midi_sysex(int leng, char * mess);
virtual int midi_metamisc(int type, int leng, char* mess);
virtual int midi_seqspecific(int type, int leng, char* mess);
virtual int midi_text(int type, int leng, char* mess);
virtual int midi_seqnum(int num);
virtual int midi_eot();
virtual int midi_keysig(int sf,int mi);
virtual int midi_tempo(long ltempo);
virtual int midi_timesig(int nn, int dd, int cc,int bb);
virtual int midi_smpte(int hr, int mn, int se, int fr,int ff);
virtual int midi_arbitrary(int leng, char* mess);
virtual int midi_time();

// write

virtual int midi_putc(char c);
virtual int midi_writetrack(int);
virtual int midi_writetempotrack();


public: //@ write new style
virtual void write_header(int format, int ntracks, int division);
virtual void write_tempo(int x);
virtual int begin_track(int trck = 0);
virtual int end_track();
virtual void write_event(int delta, int chan, int type, const  char* data, int size);
virtual void write_meta(int delta, int type, const  char* data, int size);


public: //@ process write -- old style
virtual void mfwrite(int format, int ntracks, int division, FILE* fp); 
virtual int mf_write_midi_event( unsigned long delta_time, unsigned int chan,
	unsigned int type, unsigned char *data, unsigned long size);
virtual int mf_write_meta_event( unsigned long delta_time,
	unsigned char type, unsigned char* data, unsigned long size );
virtual void mf_write_tempo( unsigned long tempo );
};

#endif
