#ifndef HUSH_ML_PARSER_H
#define HUSH_ML_PARSER_H

#include <hush/export.h>
#include <hush/term.h>
#include <hush/iter.h>
#include <iostream.h>

class scanner;

class HUSH_API parser : public iter<term> {
protected:

parser();

public:

parser(void*);        // for scanner inheritance
parser(const char* k);

parser(scanner*);     // to assign scanner and token iter
parser(iter<term>* l); // to assign (arbitrary) token iter
~parser();

virtual void init(istream* in=&cin, ostream* out=&cout);
virtual parser* create(istream* in, ostream* out);

virtual char* category() const;

virtual term* operator()();

virtual int lineno();
virtual char* error();

public: //@ to declare

virtual void op(const char* k, const char* r = 0, int i = 0);
virtual void left(const char* k, const char* r = 0, int i = 0);
virtual void right(const char* k, const char* r = 0, int i = 0);
virtual void pre(const char* k, const char* r = 0, int i = 0);
virtual void post(const char* k, const char* r = 0, int i = 0);

virtual void specifier(const char* k, const char* r = 0, int i = 0);
virtual void declarative(const char* k, const char* r = 0, int i = 0);
virtual void qualifier(const char* k, const char* r = 0, int i = 0);

virtual void modality(const char* k, const char* r = 0, int i = 0);
virtual void meta(const char* k, const char* r = 0, int i = 0);

virtual void stop(const char* k, const char* r = 0, int i = 0);
virtual void command(const char* k, const char* r = 0, int i = 0);
virtual void atom(const char* k, const char* r = 0, int i = 0);
virtual void type(const char* k, const char* r = 0, int i = 0);

virtual void end(const char* k, const char* r = 0, int i = 0);

virtual void alias(const char* k, const char* r, int i = 0);

public: //@ looks more like protected

parser* _scanner;
parser* _parser;

handler* _garbage;
char* _error;

virtual int yylex();

private: //@ but the body() is private
parser* _body();

public:  //@ and streams are public again
istream* yyin;
ostream* yyout;
ostream* yyerr;

int yytchar;
int yylineno;
int yycontext;


protected: //@ the rest is protected
iter<term>* yyiter;
term* yylval;
term* yyval;

public: //@ default

static int initialized;
static int initialize();

static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static parser* lookup(const char*);
static void destroy();

};
#endif
