#ifndef HUSH_ML_SCANNER_H
#define HUSH_ML_SCANNER_H

#include <hush/export.h>
#include <hush/ml/parser.h>

#include <hush/term.h>

class HUSH_API scanner : public parser { // iter<term> {
public:

scanner(scanner* x);
scanner(istream* in = &cin, ostream* out = &cout);

virtual ~scanner();

virtual term* operator()();

public: //@ conveniences

virtual char* get(const char* k, int* n); // to get token value

virtual char* value(const char* k);
virtual term* token(int tok, const char* s, term* v = 0);

public: //@ to determine token numbers 

virtual int _op(); // base -- must be defined in the scanner !
virtual int _left();
virtual int _right();
virtual int _pre();
virtual int _post();

virtual int _specifier();
virtual int _declarative();
virtual int _qualifier();

virtual int _modality(); 
virtual int _meta(); 

virtual int _stop(); 
virtual int _command(); 
virtual int _atom(); 
virtual int _type(); 

virtual int _end(); 

public: //@ to put

virtual void op(const char* k, const char* r = 0, int i = 0);
virtual void left(const char* k, const char* r = 0, int i = 0);
virtual void right(const char* k, const char* r = 0, int i = 0);
virtual void pre(const char* k, const char* r = 0, int i = 0);
virtual void post(const char* k, const char* r = 0, int i = 0);

virtual void specifier(const char* k, const char* r = 0, int i = 0);
virtual void declarative(const char* k, const char* r = 0, int i = 0);
virtual void qualifier(const char* k, const char* r = 0, int i = 0);

virtual void modality(const char* k, const char* r = 0, int i = 0);
virtual void meta(const char* k, const char* r = 0, int i = 0);

virtual void stop(const char* k, const char* r = 0, int i = 0);
virtual void command(const char* k, const char* r = 0, int i = 0);
virtual void atom(const char* k, const char* r = 0, int i = 0);
virtual void type(const char* k, const char* r = 0, int i = 0);

virtual void end(const char* k, const char* r = 0, int i = 0);

virtual void alias(const char* k, const char* r, int i = 0);

public: //@ to get

virtual char* op(const char* k, int* n);
virtual char* left(const char* k, int* n);
virtual char* right(const char* k, int* n);
virtual char* pre(const char* k, int* n);
virtual char* post(const char* k, int* n);

virtual char* specifier(const char* k, int* n);
virtual char* declarative(const char* k, int* n);
virtual char* qualifier(const char* k, int* n);

virtual char* modality(const char* k, int* n);
virtual char* meta(const char* k, int* n);

virtual char* stop(const char* k, int* n);
virtual char* command(const char* k, int* n);
virtual char* atom(const char* k, int* n);
virtual char* type(const char* k, int* n);

virtual char* end(const char* k, int* n);

virtual char* alias(const char* k, int* n = 0);

private: //@ ...
scanner* _body();
void* _state;

public: //@ defaults

static int initialized;
static int initialize();

static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static scanner* lookup(const char*);
static void destroy();
};

#endif
