// http_client.h
//
// header file for the HTTP netclient, using HTTP/1.0
// http_client is a child of netclient
// based on Jacco's http_client
//
// October 1995
// Bastiaan Sch\"onhage

#ifndef HTTP_CLIENT_H
#define HTTP_CLIENT_H

#include <hush/net/client/client.h>
#include <hush/net/client/url.h>

const int max_type_length = 16;

class http_client : public client {
      public:
	http_client(int tracelevel=0);
	virtual ~http_client();
	const char* type(void) const;

	// Build request:
	void get(const char* url);
	void header(const char* att, const char* val);// Adds <att>: <val> field
	void clear_header();			// Delete all header lines
	virtual bool send_request();

	
	// This is more convenient for proxy use
	virtual bool send_request(const char* host, int port, 
				  const char* request, int length); 


	// Reply:
	bool get_reply(char* &reply, int &length);
	bool get_reply_header(char* &reply, int &length);
	bool get_reply_body(char* &reply, int &length);
	bool get_reply_trailer(char* &reply, int &length);


	// Alternative reply,
	// use get_reply() first
	bool get_reply();		// use this first
	bool get_reply_body(int &fd);	// body can be read using fd 

	
	// parse reply, 
	// use get_reply() or get_reply_*(char*, int&) first
	MIME* media();
	int length(void);
	int errorcode(void);
	
	virtual void trace(int level=1);
	virtual int tracelevel() const;
	
      protected:
      private:
	char *reply_, *reply_header_;
	char *request_header_;
	MIME mime_;
	bool got_reply_, parsed_reply_, sd_is_body_;

	url* url_;
	int sd_;
	int reply_length_, reply_header_length_, content_length_;
	int max_reply_length_;
	int errorcode_;
	int tracelevel_;
	
	bool parse_reply(void);
	void skip(char* &ptr, char c);
	char* getline(char* line);
	virtual void clean_up();
};


#endif
