////////////////////////////////////////////////////////////////////////
//                                                                    //
// defs.h                                                             //
//                                                                    //
// extends 'defs.h' by the booleans OK and FALSE 		      //
//                                                                    //
// (c) 1995 D. Bolier, A. Eliens                                      //
//                                                                    //
////////////////////////////////////////////////////////////////////////
//
// jrvosse, june 12, 1996:
// Sim did use OK and FALSE as return values.
// This was an errorprone habit, since other hush packages are using
// true/false and OK/ERROR. To make things even more complicated,
// Sim defines OK to be 1, and hush defines OK as 0. Because
// functions returning OK are treated as booleans: 
//	if (empty()) { } instead of
//	if (empty() == OK) {} 
// this is likely to be a source of annoying errors.
//
// To avoid these problems, I propose the following convention:
// -  Use bool,true and false as defined in <hush/bool.h> for boolean
//    return values
// -  Use OK/ERROR to indicate succes/failure of operator()() functions.

#ifndef __DEFS_H
#define __DEFS_H

#include <hush/export.h>
#include <stddef.h>

#include <hush/bool.h>
#include <hush/ok.h>

// #undef FALSE
// #define FALSE 0	// you should use "false" instead
// #undef OK
// #define OK 1		// this should be 0, as in hush
// #undef ERROR
// #define ERROR -1	// this should be 1, as in hush

#ifndef PI
#define PI 3.1415927
#endif


#endif
