////////////////////////////////////////////////////////////////////////
//                                                                    //
// screen.h                                                           //
//                                                                    //
// implementation of the screen class, it is identical to the canvas  //
// except for the use of world coordinates.			      //
//                                                                    //
// (c) 1995 D. Bolier, A. Eliens                                      //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#ifndef __SCREEN_H
#define __SCREEN_H

#include <hush/export.h>
#include <hush/sim/hush.h>

class HUSH_API screen : public canvas
{
public :
  screen(const char* p, const char* options = "");
  screen(const widget* w, const char* p, const char* options = "");
  virtual ~screen();
  item* line(int x1,int y1,int x2,int y2,char* options = "");
  item* line(char* linespec,char* options = "");
  item* oval(int x1,int y1,int x2,int y2,char* options = "");
  item* circle(int x,int y,int r,char* options = "");
  item* polygon(char* linespec,char* options = "");
  item* rectangle(int x1,int y1,int x2,int y2,char* options = "");
  item* square(int x,int y,int r,char* options = "");
  item* bitmap(int x,int y,char* bitmap,char* options = "");
  item* window(int x,int y,widget* w,char* options = "");
  item* text(int x,int y,char* txt,char* options = "");
  void move(int id,int x,int y);
  void move(char* tg,int x,int y);
  void del(int id);
  void del(char* tg);
  void tag(int id,char* tg);
  void tag(item* id,char* tg);
  char* tags(int id);
  item* overlapping(int x,int y);
  char* items(int x,int y);
  void postscript(char* fn);

protected :
  void configure(const char* options);   // to configure the screen after

private :
  canvas* c;
  double factor;    // world / screen coordinates
  char* font;
  char* parse(const char* options);
};

#endif
