#ifndef _DVPOINTER_H
#define _DVPOINTER_H

// TODO  - a lot

#include <hush/handler.h>

#include <hush/string.h>
#include <hush/list.h>
#include <hush/dictionary.h>

#include <hush/export.h>

template <class T>
class HUSH_API dvpointer {
public:
dvpointer( const T* it, const char* s = 0 ) { 
     	_me = _this = _body = _self = (T*) it; /// all the same
	_role = 0; _inner = 0; _ctx = 0; /// the context is unknown
	_dict = 0; _list = 0;
	_mode = 0; /// do we need explicit modalities?
	_name = 0; if (s) _name = new dvstring(s);
	if (_name) *handler::_err << "Creating " << (char*) *_name << endl;
	}

~dvpointer() {
	if (_dict) delete _dict;
	if (_list) delete _list;
	if (_name) delete _name;
	}

//// Its name

char* name() { return _name?(char*)*_name:0; }

//// The facets of dvpointer objects

T* outer() { return _this; }

void body( const T* it ) { _body = it; }
T* body()  { return _role?_role:_body; }

void redirect( const T* it ) { _self = (T*) it; } 
T* self() { return _role?_role:(exists()?_self->self():_this); }

void reset() { } 
T* inner() { return _role?_role:(_inner?_inner:(_inner=self())); }

//// Invocation context

T* context( const T* it ) { T* tmp = _ctx; _ctx = it; return tmp; }
T* context() const { return _ctx?_ctx:_this; }

//// Role switching

T* become( const T* it ) { return _role = (T*) it; }
T* become( const char* nm) {
	_role =  _dict?(T*)(*_dict)[(char*)nm]:(T*)0;
	if (!_role)
	       *handler::_err << "WARNING: invalid role switch" << endl;
	return _role;
	}
	
T* become( const char* nm, const T* it) {
	_role = (T*) declare(nm,it);
	return _role;
	}

void restore() { _role = 0; }

T* declare( const char* nm, const T* it ) {
	if (!_dict) 
		_dict = new dictionary<T>;
	if (it) (*_dict)[(char*)nm] = (T*) it;
	return (T*) it;
	}

//// To register and handle components

void _register( const T* c, int flags = 1 ) {
	if (!_list)
	   ((dvpointer<T>*)this)->_list = new dvlist<T>;
	_list->insert( c );
	if (flags)_list->_register(c);
	}

//// To iter over components or repertoire

iter<T>& components() { if (!_list) /*_register(new T)*/; return *_list; }
iter<dvstring>& repertoire() { if (!_dict) declare("0",0); return *_dict; }

//// Modalities -- are these really needed

int mode( int i ) { int tmp = _mode; _mode = i; return tmp; }
int mode() const { return _mode; }

int exists() { return (_role || (_self && _self != _this))?1:0; }

private:
int _mode; /// since you never know about modalities
dvstring* _name; /// its name
T* _me;   /// the origin
T* _role; /// and what it appears to be
T* _ctx; /// the context which is _this if unknown
T* _this; /// the outer component
T* _body; /// its body
T* _self; /// the real (inner) component
T* _inner; /// optimized self
dictionary<T>* _dict;
dvlist<T>* _list; /// to process, register and collect
};

#ifndef pointer
#define pointer dvpointer
#endif

#endif
