/*
 * version 1.8
 * dvregex is a class that encapsulates the Regular expression
 * stuff. Hopefully this means I can plug in different regexp
 * libraries without the rest of my code needing to be changed.
 * Written by Jim Morris,  jegm@sgi.com
 */
#ifndef	_REGEXP_H
#define _REGEXP_H
//#include    <iostream.h>
#include    <stdlib.h>
#ifndef __FreeBSD__
#include    <malloc.h>
#endif /* __FreeBSD__ */
#include    <string.h>
#include    <assert.h>
#include    <ctype.h>

#ifndef assert
#define assert(X) 
#endif

#include    <hush/sys/regex.h>

/*
 * Note this is an inclusive range where it goes
 * from start() to, and including, end()
 */
class dvrange
{
private:
    int st, en, len;
    
public:
    dvrange()
    {
	st=0; en= -1; len= 0;
    }
    
    dvrange(int s, int e)
    {
	st= s; en= e; len= (en - st) + 1;
 	assert(st <= en && st >= 0); 
    }

    dvrange(dvrange& r) { st = r.st; en = r.en; len = r.len; }

    // test validity of the range
    operator void *() { return (st <= en && st >= 0)? this : 0;}
    int start(void) const { return st;}
    int end(void) const { return en;}
    int length(void) const { return len;}

    void set(int a, int b)
    {
	st= a; en= b; len= (en - st) + 1;
 	assert(st <= en && st >= 0); 
    }

    int operator<(const dvrange& r) const // for sorting
    {
    	return ((st == r.st) ? (en < r.en) : (st < r.st));
    }

    // x++ operator extends end of range by one
    void operator++(int){ en++; len++; }
    
    // ++x operator extends start of range by one
    void operator++(void){ st++; len--; }

#if 0
    // Don't need these yet
    dvrange operator&(const dvrange& r) const // returns intersection of two ranges
    {
        dvrange ret;
    	if(en >= 0 && r.en >= 0 && (st <= r.en) && (en >= r.st)){  // if any overlap
            ret.st= (st > r.st) ? st : r.st;
            ret.en= (en < r.en) ? en : r.en;
        }
        return ret;
    }
    
    dvrange operator|(const dvrange& r) const // returns union of two ranges if consecutive
    {
        dvrange ret;
        if(en >= 0 && r.en >= 0 && (st <= r.en+1) && (en >= r.st-1)){  // if any overlap or contiguous
            ret.st= (st < r.st) ? st : r.st;
            ret.en= (en > r.en) ? en : r.en;
        }
        return ret;
    }
#endif

    /*
    friend ostream& operator<<(ostream& os, const dvrange& r)
    {
    	os << r.st << " - " << r.en << " (" << (r.en - r.st)+1 << ")";
        return os;
    }*/
};

class dvregex
{
public:
    enum options {def=0, nocase=1};
    
private:
    regexp *repat;
    const char *target; // only used as a base address to get an offset
    int res;
    int iflg;
#ifndef	__TURBOC__
    void strlwr(char *s)
    {
	while(*s){
	    *s= tolower(*s);
	    s++;
	}
    }
#endif    
public:
    dvregex(const char *rege, int ifl= 0)
    {
        iflg= ifl;
        if(iflg == nocase){ // lowercase fold
            char *r= new char[strlen(rege)+1];
            strcpy(r, rege);
            strlwr(r);
            if((repat=regcomp(r)) == NULL){
	        //*handler::_err << "regcomp() error" << endl;
	        exit(1);
            }
            delete [] r;
	}else{
	    if((repat=regcomp (rege)) == NULL){
	        //*handler::_err << "regcomp() error" << endl;
	        exit(1);
	    }
        }
    }
    
    ~dvregex()
    {
	free(repat);
    }    

    int match(const char *targ)
    {
        int result;
    	if(iflg == nocase){ // fold lowercase
            char *r= new char[strlen(targ)+1];
            strcpy(r, targ);
            strlwr(r);
            result = regexec(repat, r); 
    	    target= r; // looks bad but is really ok, really
            delete [] r;
        }else{
	    result = regexec(repat, targ);
	    target= targ;
	}

	return ((result == 0) ? 0 : 1);
    }
    
    int groups(void) const
    {
	int result= 0;
	for (int i=0; i<NSUBEXP; i++) {
	    if(repat->startp[i] == NULL) break;
	    result++;
	}
	return result;
    }
    
    dvrange getgroup(int n) const
    {
	assert(n < NSUBEXP);
	return dvrange((int)(repat->startp[n] - (char *)target),
		     (int)(repat->endp[n] - (char *)target) - 1);
    }
};
#endif
