#ifndef CANVAS_H
#define CANVAS_H

#include <widgets/export.h>

class handler;
class canvas; 

#include <hush/widget.h>
#include <hush/item.h>


class WIDGETS_API canvas : public widget {
public:

  char* type() const;  /// canvas, what else
  
  canvas();

  canvas(const char* p, const char* opts = "", canvas* = 0);
  canvas(const widget* w, const char* p, const char* ="", canvas* = 0);

  virtual widget* _create(const char* p, const char* opts = "",
				widget* = 0);
  virtual widget* _create(const widget* w, const char* p,
				const char* opts = "", widget* = 0);

  virtual ~canvas();

  virtual void region(int x1, int y1, int x2, int y2 );
  
  enum { window_coords, canvas_coords }; // window_coords is default

  virtual void coordsystem(int);
  virtual int coordsystem() const;

  virtual int canvasx(int screenx) const;
  virtual int canvasy(int screeny) const;

  virtual char* coords(int id) const;
  virtual char* coords(const char* id) const;

  virtual char* bbox(int id) const;
  virtual char* bbox(const char* id) const;
  
  virtual void tag(int id, const char* tag);
  virtual char* tags(int id) const;
  
  virtual void del(int id);
  virtual void del(const char* id);
  
  virtual void move(int id, int x, int y);
  virtual void move(const char* id, int x, int y);
  
  virtual item* bitmap(int x1, int y1, const char* bitmap, const char* options="");
  virtual void image(const char*) { } // to silence the compiler
  virtual item* image(int x1, int y1, const char* bitmap, const char* options="");
  virtual item* photo(int x1, int y1, int x2, int y2, const char* options ="");
  virtual void readppm(int id, const char* file ="");
  
  virtual item* line(int x1, int y1, int x2, int y2, const char* options ="");
  virtual item* line(const char* linespec, const char* options ="");
  virtual item* arc(int x1, int y1, int x2, int y2, const char* options ="");
  virtual item* oval(int x1, int y1, int x2, int y2, const char* options ="");
  virtual item* circle(int x1, int y1, int r, const char* options ="");
  virtual item* polygon(const char* linespec, const char* options ="");
  virtual item* rectangle(int x1, int y1, int x2, int y2, const char* options ="");
  virtual item* text(int x1, int y1, const char* txt, const char* options="");
  virtual item* window(int x1, int y1, const char* win, const char* options="");
  virtual item* window(int x1, int y1, widget* win, const char* options="");
  
  virtual void raise(const char* tag, const char* abovethis="");
  virtual void lower(const char* tag, const char* belowthis="");
  
  /// obsolete or experimental
  
  item* sphere(int x1, int y1, int x2, int y2, const char* color="white", const char* tag = "sphere", const char* options ="");
  item* sphere(int x1, int y1, int r, const char* color="white", const char* tag = "sphere", const char* options ="");
  
  /// item* methods
  
  virtual item* current();
  virtual item* overlapping(int x, int y, const char* options);
  virtual item* overlapping(int x, int y);
  
  virtual char* items(int x, int y);
  virtual char* items(int x1, int y1, int x2, int y2);
  
  virtual void itemconfigure(int it, const char* options);
  virtual void itemconfigure(const char* tag, const char* options);
  
  virtual int itembind(int i, const char* s, dvbinding* a, const char* args = "" );
  virtual int itembind(const char *i, const char* s, dvbinding* a, const char* args = "" );
  
  virtual void fontmap(const char *xfont, const char *printerfont, int ptsize);
  virtual void postscript(const char* file, const char* options="");

protected:
  void install(dvbinding* ac, const char* args = "");
  
protected:
  virtual int itemdirect(int i, const char *s, dvbinding* a, const char* args="" );
  virtual int itemdirect(const char* i, const char *s, dvbinding* a, const char* args="" );

  virtual int convertx(int) const;
  virtual int converty(int) const;

public: 
	canvas* self() const;

	canvas* _body() const; // virtual
	canvas* _context() const;
	canvas* _self() const;
private: 
	canvas* _bdy;
	int _exists() const;
	canvas* _this() const; // non-virtual

public: // factory

static canvas* _default;
static canvas* get_default();
static void set_default(canvas*);

static void* table;
static void declare(const char*, void*, int flag = 0);
static canvas* lookup(const char*);

public: 
static int created;
static int deleted;
};

#endif
