#ifndef LISTBOX_H
#define LISTBOX_H

#include <widgets/export.h>

#include <hush/widget.h>

class WIDGETS_API listbox : public widget {
public:

  char* type() const ;
  
  listbox();
   
  listbox(const char* p, const char* opts = "", listbox* = 0);
  listbox(const widget* w, const char* p, const char* ="", listbox* = 0);
   
  widget* _create(const char* p, const char* opts = "", widget* = 0);
  widget* _create(const widget* w, const char* p,
   				 const char* opts = "", widget* = 0);
   
  virtual ~listbox();
  
  virtual void insert(const char*);		// insert at end
  virtual void insert(int index, const char*);// insert at index

  virtual void del();				// delete all
  virtual void del(int index);		// delete one 
  virtual void del(int first, int last);	// delete range

  virtual void see(int index);		// make entry visible
 
  virtual char* get(int index) const;		// get entry 
  virtual int size() const;			// # entries

  virtual void select();			// select all
  virtual void select(int first, int last);	// select range
  virtual void unselect();			// unselect all
  virtual void unselect(int first, int last);	// unselect range

  virtual int selected(int index) const;	// true if entry selected
  virtual int selection() const;		// get index of (first) 
						//    selected entry
  // todo: geometry, scanning, selecting
  
  virtual void singleselect();
  
protected: // virtual?
  void install(dvbinding* ac, const char* args );
  void init(const char* options);

public: 
	listbox* self() const;

	listbox* _body() const; // virtual
	listbox* _context() const;
	listbox* _self() const;
private: 
	listbox* _bdy;
	int _exists() const;
	listbox* _this() const; // non-virtual

public: // factory

static listbox* _default;
static listbox* get_default();
static void set_default(listbox*);

static void* table;
static void declare(const char*, void*, int flag = 0);
static listbox* lookup(const char*);

public: 
static int created;
static int deleted;
};

#endif
