////////////////////////////////////////////////////////////////////////
//                                                                    //
// handlers.c                                                         //
//                                                                    //
// implementation of the diverse handlers                             //
//                                                                    //
// (c) 1995 D. Bolier, A. Eliens                                      //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#include <hush/source.h>
#include <hush/sim/handlers.h>

#ifdef HUSH
#include <string.h>
#include <hush/sim/hush.h>
#include <hush/sim/defs.h>


// constructor, initializes the start variabele.

buttonhandler::buttonhandler()
{
  start = true;
}


// redefines the handler::operator()() function, it sets the start
// variabele to its opposite

int buttonhandler::operator()()
{
  if (start) start = false;
  else start = true;
  return OK;
}

#else

buttonhandler::buttonhandler()
{
}

#endif

#ifdef HUSH

#include <hush/sim/simulation.h>

quithandler::quithandler(control* co)
{
  c = co;
}


// redefines the handler::operator()() function, it invokes the simulation::
// quit function

int quithandler::operator()()
{
  // simulation pointer exists
  thesimulation() -> quit();
  return OK;
}

#else

quithandler::quithandler()
{
}

#endif

#ifdef HUSH

exithandler::exithandler(control* co)
{
  c = co;
}


// redefines the handler::operator()() function, it invokes kit::quit
// function

int exithandler::operator()()
{
  // simulation pointer exists
  if (thekit()) thekit() -> quit();
  return OK;
}

#else

exithandler::exithandler()
{
}

#endif

#ifdef HUSH

// constructor, initializes the speed variabele, the factor and the
// scale widget

scalehandler::scalehandler(scale* s,double f)
{
  sc = s;
  sp = 0;
  fc = f;
}


// redefines the handler::operator()() function, it sets the fraction
// of the delay time. The simulation::selectnext function will sleep
// for this fraction times the interactivation time in seconds.

int scalehandler::operator()()
{
  if (simulation::hush_enabled) {
  sp = (double)(100 - sc -> get()) / 100 * fc;
  }
  return OK;
}

#else

scalehandler::scalehandler()
{
}

#endif

#ifdef HUSH

#include <stdlib.h>

// constructor, initializes the entry value and widget

entryhandler::entryhandler(entry* en)
{
  e = en;
  input = new char[10]; input[0]=0;// jrvosse
}


// redefines the handler::operator()() function, it sets the entry
// value

int entryhandler::operator()()
{
  strcpy(input, e -> get());
  return OK;
}

#else

entryhandler:: entryhandler()
{
}

#endif
