////////////////////////////////////////////////////////////////////////
//                                                                    //
// Simulation.c                                                       //
//                                                                    //
// the simulation class, contains the main simulation routine, that   //
// is responsible for the activation of the pending and conditional   //
// events, and the methods for manipulating the simulation environment//
//                                                                    //
// (c) 1995 D. Bolier, A. Eliens                                      //
//                                                                    //
////////////////////////////////////////////////////////////////////////


// GNU forgot so
//extern "C" int gettimeofday(...);

#include <hush/source.h>
#include <hush/sim/kit.h>

#include <time.h>
#include <hush/sys/time.h>

#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <hush/sys/unistd.h>             // bas
#include <string.h>
#include <math.h>

#ifdef BORLAND
#define TIMEZONE __timezone
#else
#define TIMEZONE timezone
#endif

#ifdef HUSH

#include <hush/sim/hush.h>

extern kit* thekit();

enum {SCREENWIDTH = 400,TEXTHEIGHT = 20};

//AE
extern void set_simevent_fun(void*);
#endif


#include <hush/sim/control.h>
#include <hush/sim/clock.h>
#include <hush/sim/event.h>
#include <hush/sim/defs.h>
#include <hush/sim/simulation.h>
#include <hush/sim/error.h>
#include <hush/sim/scheduler.h>
#include <hush/sim/conditionallist.h>
#include <hush/sim/tertiarytree.h>
#include <hush/sim/reports.h>
#include <hush/sim/histogram.h>
#include <hush/sim/converter.h>

#ifdef HUSH
static event* sim_event(int q, int s, int p) {
	return new simevent(q,s,p);
	}
#endif

static simulation* sim = 0;   // the simulation object

// invoked in other classes of the library

extern simulation* thesimulation()
{
  return sim;
}

// check for hush

int simulation::hush_enabled = 0;
  

// constructor, allocate and initialize the various data members

simulation::simulation(int enable)
#ifdef HUSH
		: kit(this)
#endif
{
  simulation::hush_enabled = enable;
  sim = this;
  simclock = 0.0;
  timestamp = 0.0;
  clockstamp = 0.0;
  running = false;
  act = false;
  allflag = true;
  reportsflag = true;
  statisticsflag = true;
  tim = 0.0;
  termcount = 0;
  countset = false;
  countreached = false;
  numbercreated = 0;
  numberterminated = 0;
  numberactivated = 0;
  resourceindex = 0;
  queueindex = 0;
  waitingindex = 0;
  waitingallindex = 0;
  eventindex = 0;
  behaviorindex = 0;
  _speed = 0; // 0 = plain mode
  _ctrlhandler = 0;
  gettimeofday(&inittime,NULL);
  for (int i=0;i<MAXREPORT;i++)
  {
    resources[i] = NULL;
    queues[i] = NULL;
    waitings[i] = NULL;
    waitingsall[i] = NULL;
  }
  events = new eventreportpointer[MAXREPORT];
  behaviors = new behaviorreportpointer[MAXREPORT];
  sched = new scheduler();
  condlist = new conditionallist();
  curr = NULL;
#ifdef HUSH
  scr = NULL;
  cnt = NULL;
  ck = NULL;
  textoptions = new char[BUFSIZ];
  sprintf(textoptions,"-fill black");
  //AE
  set_simevent_fun( sim_event );
#endif
cerr << "Simulation (hush=" << hush_enabled << ")" << endl;
}
  

// destructor, dereference histograms and deallocate memory

simulation::~simulation()
{
  delete sched;
  delete condlist;
  curr = NULL;
  for (int i=0;i<MAXREPORT;i++)
  {
    resources[i] = NULL;
    queues[i] = NULL;
    waitings[i] = NULL;
    waitingsall[i] = NULL;
  }
  if (events)
//    delete events;  // NO      <----------
    delete[] events;  // YES     <----------
  if (behaviors)
//    delete behaviors;  // NO      <----------
    delete[] behaviors;  // YES     <----------
#ifdef HUSH
  if (scr) delete scr; scr=NULL;
  cnt = NULL;
  ck = NULL;
  delete textoptions;
#endif
}


// format screen

screen* simulation::format(char* p,char* options)
{
#ifdef HUSH
  scr = hush_enabled?(new screen(p,parse(options))):0;
  return scr;
#else
  return NULL;
#endif
}
  
screen* simulation::format(widget* w,char* p,char* options)
{
#ifdef HUSH
  scr = hush_enabled?(new screen(w,p,parse(options))):0;
  return scr;
#else
  return NULL;
#endif
}


#ifdef HUSH

// sets the realtime mode and speed, the scan flag
// and the screen options

char* simulation::parse(char* options)
{
  int bg,fg,wd,hg;

  // standard options
  fg = bg = 0;
  wd = SCREENWIDTH;
  hg = 6*TEXTHEIGHT;

  // parse options
  converter* cnv = new converter(options);
  for (int i=1; i <= cnv -> argc(); i++)
  {
    if ((!strcmp((*cnv)[i],"-foreground")) && (i < cnv -> argc()))
      fg = i+1; 
    else if ((!strcmp((*cnv)[i],"-background")) && (i < cnv -> argc()))
      bg = i+1; 
    else if ((!strcmp((*cnv)[i],"-width")) && (i < cnv -> argc()))
      wd = atoi((*cnv)[i+1]); 
    else if ((!strcmp((*cnv)[i],"-height")) && (i < cnv -> argc()))
      hg = atoi((*cnv)[i+1]); 
  }

  // format
  if (fg)
    sprintf(textoptions,"-fill %s",(*cnv)[fg]);
 
  char* screenoptions = new char[BUFSIZ];
  if (bg)
    sprintf(screenoptions,"-width %d -height %d -worldx %d -worldy %d -background %s",wd,hg,SCREENWIDTH,6*TEXTHEIGHT,(*cnv)[bg]);
  else
    sprintf(screenoptions,"-width %d -height %d -worldx %d -worldy %d -background white",wd,hg,SCREENWIDTH,6*TEXTHEIGHT);
  delete cnv;
  return screenoptions;
}

#endif


// initialize

void simulation::init(char* options)
{
  // parse options
  converter* cnv = new converter(options);
  for (int i=1; i <= cnv -> argc(); i++)
  {
    if ((!strcmp((*cnv)[i],"-realtime")) && (i < cnv -> argc()))
      _speed = atof((*cnv)[i+1]);
    else if ((!strcmp((*cnv)[i],"-scan")) && (i < cnv -> argc()))
      if (!strcmp((*cnv)[i+1],"yes"))
        allflag = false;
      else if (!strcmp((*cnv)[i+1],"no"))
        allflag = true;
      else
        sim_err(SCAN);
    else if ((!strcmp((*cnv)[i],"-suppress")) && (i < cnv -> argc()))
      if (!strcmp((*cnv)[i+1],"none"))
      {
        reportsflag = true;
        statisticsflag = true;
      }
      else if (!strcmp((*cnv)[i+1],"reports"))
      {
        reportsflag = false;
        statisticsflag = true;
      }
      else if (!strcmp((*cnv)[i+1],"statistics"))
      {
        reportsflag = true;
        statisticsflag = false;
      }
      else if (!strcmp((*cnv)[i+1],"both"))
      {
        reportsflag = false;
        statisticsflag = false;
      }
      else
        sim_err(SUPPRESS);
  }
  delete cnv;
}


// defaults for the initialisation

void simulation::suppress()
{
  reportsflag = statisticsflag = false;
}

void simulation::scan()
{
  allflag = true;
}

void simulation::realtime(double sp)
{
  // store the timestamp and clock to be able to change speed during run
  timestamp = realtime();	// bas
  clockstamp = simclock;	// bas
  _speed = sp;
}


// returns real time elapsed

double simulation::realtime() 
{
  // set actual starting time
  struct timeval tv;
  struct TIMEZONE tz;
  gettimeofday(&tv,&tz);
  //long start = tv.tv_sec;
  long delta_sec = tv.tv_sec - inittime.tv_sec;
  long delta_usec = tv.tv_usec - inittime.tv_usec;
  double curacttime =  (double) delta_sec + 
    (double) delta_usec / (double) 1000000L;
  return curacttime;
}
 
// I have changed the three (non-functioning) run's
// into a single run2 member.
// The old run members will kept as the public interface
// bastiaan, Feb. 1998

// run until no events left

void simulation::run()
{
  run2(0, 0, 0);
}

// run the simulation for t time units

void simulation::run(double t)
{
  run2(t, 0, 0);
}

// run the simulation for count terminated events

void simulation::run(int count, int killflag)
{
  run2(0, count, killflag);
}

// the new run2 mainloop that runs for maxtime, maxcount
// or till no events are left
// a maxtime=0 or maxcount=0 implies no limit on time resp. count
// run2 by bastiaan, Feb. 1998

void simulation::run2(double maxtime, int maxcount, int killflag)
{
  // quit if we don't have any events
  if (sched->empty()) {
	reports(NOLEFT,0 , tim);
	return;
  }

  // running flag
  running = true;
  int schedule = true;

  // set actual starting time
  struct timeval tv;
  struct TIMEZONE tz;
  gettimeofday(&tv,&tz);
  long start = tv.tv_sec;
  double st = tim;

  // keep stamps of current realtime and simulationclock
  timestamp = realtime();
  clockstamp = simclock;

  // set the duration
  if (maxtime>0) {
    dur = maxtime;            // why ??
    maxtime += simclock;      // is this really changed ??
  }

  // do we have to check for killflag counts ??
  if ( (maxcount > 0) && killflag ) {
    countset = true;
    termcount = maxcount;
  }

  // main routine
  // changed to a working (non-busywaiting) realtime loop by bastiaan
  while ( running ) {	

#ifdef HUSH
    // check the start/stop button
    if ( (!hush_enabled) || (!cnt) || cnt -> check() )
    {
#endif  
      
      if (_speed)  // realtime
      {
	// sleep if necessary in realtime
	// wait until realtime = timestamp + (eventtime - clockstamp)/speed
	double curacttime = realtime();			
	double nexteventtime = timestamp + 
		(sched->giveroot()->time() - clockstamp) / _speed;
	if (nexteventtime > curacttime) { 
		double st = (nexteventtime - curacttime) * 1000000.0;
		unsigned long sleeptime = (unsigned long) st;
		// sleep only a maximum of _maxdelay microseconds
		if (sleeptime > _maxdelay) sleeptime = _maxdelay;
		usleep ( sleeptime );
	}		
	if (nexteventtime > realtime()) schedule=false;
	else schedule = true;
      }

      // call the control henadler if available
      if (_ctrlhandler) _ctrlhandler->dispatch(0);
      // is is safe to use a 0 event ???


      // we could have slept only partially
      // so only schedule events if necessary 
      if (schedule) {
	// the main part of the run loop
	selectnext();
	execute();
	scanconditional();
      }

#ifdef HUSH
    }
    else if (hush_enabled && cnt)
      cnt -> check();
#endif

      // check if we have to quit the loop
      if (sched->empty()) running=false;
      if ( (maxtime>0) && (tim >= maxtime) ) running=false;
      if ( (maxcount>0) && (!killflag) && (numberterminated >= maxcount) ) 
	running=false;
      if ( (maxcount>0) && (killflag) && (countreached) ) running=false;
  }


  // get the end time
  gettimeofday(&tv,&tz);
  long end = tv.tv_sec;

  // how did we stop ?
  if ( (maxtime>0) && (tim >= maxtime) )
    reports(END, end-start,st);
  else if ( (maxcount>0) && (!killflag) && (numberterminated >= maxcount) )
    reports(COUNT, end-start, st);
  else if ( (maxcount>0) && (killflag) && (countreached) )
    reports(COUNT, end-start, st);
  else if (sched -> empty())
    reports(NOLEFT,end-start,st);
  else if (!running)
    reports(QUIT,end-start,st);
  else // bastiaan: this will never happen in our case, maybe using hush ??
    reports(REALTIME,end-start,st);

  // may be a next run, so 

  numbercreated = 0;
  numberterminated = 0;
  numberactivated = 0;
}

// set controlhandler by bas
void simulation::ctrlhandler(handler *h, unsigned long maxdelay)
{
	_ctrlhandler = h;
	_maxdelay = maxdelay;
}

// get controlhandler by bas
handler* simulation::ctrlhandler()
{
	return _ctrlhandler;
}

// end of changes by bastiaan, Feb. 1998

// print the reports and a screen with statistics (if HUSH/tk are defined)
// depending on the flags

void simulation::reports(stoptype type,long tm,double start)
{
if (statisticsflag) // suppressed ?
{
#ifndef HUSH
  start = start;  
#endif

  // statistics of actual time taken, number terminated, etc..
  char* buf = new char[BUFSIZ];
#ifdef HUSH
  if (scr)
    scr -> del("simulation");
  else if (hush_enabled)
  {
    ::toplevel* tl = new ::toplevel(".simulation");
    scr = new screen(tl,".screen","-background white");
    scr -> geometry(SCREENWIDTH,6*TEXTHEIGHT);
    thekit() -> pack(scr);
  }
#endif

  sprintf(buf,"Simulation started at : %f",start);

#ifdef HUSH
  item* it;
  if (hush_enabled) {
  it = scr -> text(SCREENWIDTH/2,TEXTHEIGHT/2,buf,textoptions);
  scr -> tag((int)*it,"simulation");
  } else
  	cout << endl << buf << endl;
#else
  cout << endl << buf << endl;
#endif

  switch (type)
  {
    case NOLEFT :
      sprintf(buf,"Simulation stopped - no scheduled events left at : %f",tim);
      break;
    case QUIT :
      sprintf(buf,"Simulation stopped - quit called at : %f",tim);
      break;
    case END :
      sprintf(buf,"Simulation stopped - end time reached at : %f",tim);
      break;
    case COUNT :
      sprintf(buf,"Simulation stopped - zero termination count at : %f",tim);
      break;
    case REALTIME :
      sprintf(buf,"Simulation stopped - simulation time > actual time %f",tim);
      break;
  }

#ifdef HUSH
  if (hush_enabled) {
  it = scr -> text(SCREENWIDTH/2,3*TEXTHEIGHT/2,buf,textoptions);
  it -> tag("simulation");
  } else
  	cout << buf << endl;
#else
  cout << buf << endl;
#endif

  sprintf(buf,"Created - %d events",numbercreated);

#ifdef HUSH
  if (hush_enabled) {
  it = scr -> text(SCREENWIDTH/2,5*TEXTHEIGHT/2,buf,textoptions);
  it -> tag("simulation");
} else
  	cout << endl << buf << endl;
#else
  cout << endl << buf << endl;
#endif

  sprintf(buf,"Activated - %d events",numberactivated);

#ifdef HUSH
  if (hush_enabled) {
  it = scr -> text(SCREENWIDTH/2,7*TEXTHEIGHT/2,buf,textoptions);
  it -> tag("simulation");
  } else
  	cout << buf << endl;
#else
  cout << buf << endl;
#endif

  sprintf(buf,"Terminated - %d events",numberterminated);

#ifdef HUSH
  if (hush_enabled) {
  it = scr -> text(SCREENWIDTH/2,9*TEXTHEIGHT/2,buf,textoptions);
  it -> tag("simulation");
  } else
  	cout << buf << endl;
#else
  cout << buf << endl;
#endif

  sprintf(buf,"Actual time taken - %d seconds",(int)tm);

#ifdef HUSH
  if (hush_enabled) {
  it = scr -> text(SCREENWIDTH/2,11*TEXTHEIGHT/2,buf,textoptions);
  it -> tag("simulation");
  } else cout << endl << buf << endl;
#else
  cout << endl << buf << endl;
#endif

//  delete buf;  // NO      <----------
  delete[] buf;  // YES     <----------

}  // end suppressed ?

  // print reports

  if (reportsflag)   // suppressed ?
  {
    int i = 0;
    while (i < resourceindex)
    {
      cout << (*resources[i]);
      i++;
    }
    i = 0;
    while (i < queueindex)
    {
      cout << (*queues[i]);
      i++;
    }
    i = 0;
    while (i < waitingindex)
    {
      cout << (*waitings[i]);
      i++;
    }
    i = 0;
    while (i < waitingallindex)
    {
      cout << (*waitingsall[i]);
      i++;
    }
    i = 0;
    while (i < eventindex)
    {
      cout << (*events[i]);
      i++;
    }
    i = 0;
    while (i < behaviorindex)
    {
      cout << (*behaviors[i]);
      i++;
    }
  }  // end suppressed ?
  
#ifdef HUSH
  if (simulation::hush_enabled && thekit()) thekit() -> update();
  //else cerr << "No kit available" << endl;
#endif
}


// quit the simulation, just set the flag

void simulation::quit()
{
  running = false;
}


// terminate the given event, it should be PASSIVE or ACTIVE

int simulation::terminate(event* e)
{
  // e should exist and should be PASSIVE or ACTIVE
  if (!e)
    sim_err(NULLEV);
  if ( (e -> _active()) || (e -> passive()) )
  {
    if (e == curr)
    {
      act = false;
      curr = NULL;
    }

    // if kill-flag decrease count and possibly quit
    if (e -> killflag())
      decreasecount();

    // if it has a report on the time spent, add a sample
    eventreport* erep = findeventreport(e -> reporttitle());
    if (erep)
      (*erep)(e);
    delete e;
    numberterminated++;

    return true;
  }
  else
    sim_warn(TERMINATE);
  return false;
}
  

// schedule an event e to be activated over t time units

int simulation::schedule(event* e,double t)
{
  // e should exist and should be PASSIVE
  if (!e)
    sim_err(NULLEV);
  if (e -> passive()) 
  {
    if (curr == e)
      act = false;
    return (sched -> insert(t,e));
  }
  else
    sim_warn(SCHEDULE);
  return false;
}


// schedule the current to be activated over t time units

int simulation::wait(double t)
{
  // the current should exist and should still be ACTIVE
  if (!curr)
    sim_err(NULLEV);
  if (curr -> _active())
  {
    act = false;
    return (sched -> insert(t,curr));
  }
  else
    sim_warn(WAIT);
  return false;
}
    

// passivate the given event, extract it from the conditional list or
// pending tree or just make it PASSIVE

int simulation::passivate(event* e)
{
  // e should exist
  if (!e)
    sim_err(NULLEV);
  // remove it from the conditional list
  if (e -> conditional() || e -> closed())
    {
    condlist -> remove(e);
    e -> state(PASSIVE);
    if (curr == e)
      act = false;

    return true;
  }
  // remove it from the pending tree
  else if (e -> pending()) 
  {
    sched -> extract(e);
    e -> state(PASSIVE);
    return true;
  }
  // just make it passive
  else if (e -> _active()) 
  {
    e -> state(PASSIVE);
    act = false;
    return true;
  }

  else
    sim_warn(PASSIVATE);
  return false;
}


// activate the given event, extract it from the conditional list or
// the pending tree and schedule it to be activated immediately

int simulation::activate(event* e)
{
  // e should exist
  if (!e)
    sim_err(NULLEV);
  // on conditional list ?
  if (e -> conditional() || e -> closed()) 
  {
    condlist -> remove(e);
    if (curr == e)
      curr -> state(ACTIVE);
    sched -> insert(0.0,e);
    return true;
  }
  // just passive ?
  else if (e -> passive()) 
  {
    sched -> insert(0.0,e);
    return true;
  }
  // on the pending tree and time to be activated true?
  else if ((e -> pending()) && (e -> time() > simclock)) 
  {
    sched -> extract(e);
    sched -> insert(0.0,e);
    return true;
  }
  else
    sim_warn(ACTIVATE);
  return false;
}


// add the event to the conditional list

int simulation::hold(event* e)
{
  // e should exist and should be PASSIVE or ACTIVE
  if (!e)
    sim_err(NULLEV);
  if (e -> passive()) 
  {
    condlist -> add(e);
    return true;
  }
  else if ( e -> _active())
  {
    condlist -> add(e);
    act = false;
    return true;
  }
  else
    sim_warn(HOLD);
  return false;
}


// it stays on the conditional list, but it's state is chanced to CLOSED
// so it can't be scanned

int simulation::withdraw(event* e)
{
  // e should exist and should be CONDITIONAL
  if (!e)
    sim_err(NULLEV);
  if (e -> conditional())
  {
    e -> state(CLOSED);
    return true;
  }
  else
    sim_warn(WITHDRAW);
  return false;
}


// enable the event to be scanned again

int simulation::reinstate(event* e)
{
  // e should exist and should be CLOSED
  if (!e)
    sim_err(NULLEV);
  if (e -> closed())
  {
    e -> state(CONDITIONAL);
    return true;
  }
  else
    sim_warn(REINSTATE);
  return false;
}


// cancel all events that receive true from the event::verify method
// return the number of terminated events

int simulation::cancel()
{
  int i = sched -> cancel();
  cerr << "**SCHED: " << i;
  i += condlist -> cancel();
  cerr << "+COND: " << i << endl;
  return i;
}


// reset the simulation environment, as if it was just created

void simulation::reset()
{
  delete sched;
  delete condlist;
  sched = new scheduler();
  condlist = new conditionallist();
  act = false;
  curr = NULL;
  simclock = 0.0;
  timestamp = 0.0;
  clockstamp = 0.0;
  running = false;
  tim = 0.0;
  termcount = 0;
  countset = false;
  countreached = false;
  numbercreated = 0;
  numberterminated = 0;
  numberactivated = 0;
  gettimeofday(&inittime,NULL);
  int i = 0;
  while (i < resourceindex)
  {
    resources[i] -> reset();
    i++;
  }
  i = 0;
  while (i < queueindex)
  {
    queues[i] -> reset();
    i++;
  }
  i = 0;
  while (i < waitingindex)
  {
    waitings[i] -> reset();
    i++;
  }
  i = 0;
  while (i < waitingallindex)
  {
    waitingsall[i] -> reset();
    i++;
  }
  i = 0;
  while (i < eventindex)
  {
    events[i] -> reset();
    i++;
  }
  i = 0;
  while (i < behaviorindex)
  {
    behaviors[i] -> reset();
    i++;
  }
#ifdef HUSH
  if (hush_enabled) {
  if (cnt)
    cnt -> reset();
  if (ck)
    ck -> setclock(0.00);
  if (scr)
    scr -> del("simulation");
  }
#endif
}


// add a report on the occupation of a resource

int simulation::addresourcehistogram(histogram* h)
{
  if (resourceindex >= MAXREPORT)
  {
    sim_warn(REPORT);
    return false;
  }
  resources[resourceindex] = h;
  resourceindex++;
  return true;
}


// add a report on the size of a queue

int simulation::addqueuehistogram(histogram* h)
{
  if (queueindex >= MAXREPORT)
  {
    sim_warn(REPORT);
    return false;
  }
  queues[queueindex] = h;
  queueindex++;
  return true;
}


// add a report on waiting times

int simulation::addwaitinghistogram(histogram* h)
{
  if (waitingindex >= MAXREPORT)
  {
    sim_warn(REPORT);
    return false;
  }
  waitings[waitingindex] = h;
  waitingindex++;
  return true;
}


// add a report on waiting times

int simulation::addwaitingallhistogram(histogram* h)
{
  if (waitingallindex >= MAXREPORT)
  {
    sim_warn(REPORT);
    return false;
  }
  waitingsall[waitingallindex] = h;
  waitingallindex++;
  return true;
}


// add a report on the time events spent in the simulation

int simulation::addeventreport(eventreport* e)
{
  if (eventindex >= MAXREPORT)
  {
    sim_warn(REPORT);
    return false;
  }
  events[eventindex] = e;
  eventindex++;
  return true;
}


// add a report on behavior

int simulation::addbehaviorreport(behaviorreport* s)
{
  if (behaviorindex >= MAXREPORT)
  {
    sim_warn(REPORT);
    return false;
  }
  behaviors[behaviorindex] = s;
  behaviorindex++;
  return true;
}


// return the behavior report with the same histogram as the one given 
// as parameter, if any

behaviorreport* simulation::findbehaviorreport(histogram* h)
{
  int i=0;
  while (i < behaviorindex )
  {
    if (h == behaviors[i] -> analyzed())
      return behaviors[i];
    i++;
  }
  return NULL;
}


// return the event report with the same title as the one given as
// parameter, if any

eventreport* simulation::findeventreport(char* tp)
{
  int i=0;
  if (tp == NULL)
    return NULL;
  while ( i < eventindex )
  {
    if (strcmp(tp,(events[i] -> histogramtitle())) == 0)
      return events[i];
    i++;
  }
  return NULL;
}
  

// select the next events from the pending tree, if any.

void simulation::selectnext()
{
#ifdef HUSH
  unsigned int delayfactor;
#endif

  // adjust the state of the event that had just been activated
  if (curr) 
  {
    if (curr -> _active())
      curr -> state(PASSIVE);
  }

  // get the first event(s), depending on the all flag
  curr = sched -> front(allflag);

#ifdef HUSH
  // sleep for 'delayfactor' milliseconds and update the clock
  // if the control and clock widgets exist
  // because the delay is fixed in the new main loop,      // bas
  // the delay by means of tk->after is not necessary      // bas
  if (cnt)
  {
    delayfactor = (unsigned int)(1000 * cnt -> speed() * (curr -> time() - simclock));
    // if (thekit()) thekit() -> after(delayfactor);  // bas
  }
  if (ck && hush_enabled)
    ck -> setclock(curr -> time());
#endif

  // adjust simulation clock and the time counter
  simclock = curr -> time();
  tim = simclock;
}


// execute the current events

void simulation::execute()
{
  event* next = curr -> middlepointer();

  // adjust state and invoke the function operator
  act = true;
  curr -> state(ACTIVE);
  curr->dispatch(curr); //AE (void)(*curr)();
  numberactivated++;

  // execute events with the same activation time,
  // the middlepointer of the last event of this list points to the
  // first event of this list 
  if (next) 
  {
    event* final = next;
    do 
    {
      curr = next;
      act = true;
      curr -> state(ACTIVE);
      next = curr -> middlepointer();
      curr->dispatch(curr); //AE (void)(*curr)();
      numberactivated++;
    } while (final != next);
  }
}


// scan the conditional list

void simulation::scanconditional()
{
  // cerr << "scanconditonal " << endl;
  event* conditionalHd = condlist -> head();
  // the list should exist
  if (conditionalHd != NULL)
  {
    event* next = conditionalHd -> rightpointer();

    // adjust the state of the current if it exists, the activated
    // event stays conditional
    act = true;
    if (curr) 
    {
      if (curr -> _active()) 
        curr -> state(PASSIVE);
    }

    // invoke the function operator of the head, if not CLOSED
    curr = conditionalHd;
    if (curr -> conditional())
    {
      curr->dispatch(curr); // AE (void)(*curr)();
      numberactivated++;
    }

    // while a next CONDITIONAL event, activate it
    while (next) 
    {
      curr = next;
      next = curr -> rightpointer();
      act = true;
      if (curr -> conditional()) 
      {
        curr->dispatch(curr); //AE (void)(*curr)();
        numberactivated++;
      }
    }
  }
}


// adjust the counter and possibly quit

void simulation::decreasecount()
{
  if (countset)
  {
    termcount--;
    if (termcount < 1)
      countreached = true;
  }
}


// generate a postscript of the screen with statistics

void simulation::postscript(char* fn)
{
#ifdef HUSH
  if (scr)
    scr -> postscript(fn);
#else
  fn = fn;
#endif
}


// put the simulation object as the scheduler and the conditional list
// in an output stream

ostream& operator<<(ostream& os,simulation& s)
{
  scheduler* sc = s.sched;
  conditionallist* c = s.condlist;
  os << (*sc);
  os << (*c);
  return os;
}

